local _G = getfenv(0);

local modName = "QButtons";
local iconSize = 32;
local slots = {
	"HeadSlot", "NeckSlot", "ShoulderSlot", "BackSlot", "ChestSlot", "ShirtSlot", "TabardSlot", "WristSlot",
	"HandsSlot", "WaistSlot", "LegsSlot", "FeetSlot", "Finger0Slot", "Finger1Slot", "Trinket0Slot", "Trinket1Slot",
	"MainHandSlot", "SecondaryHandSlot", "RangedSlot",
};

QButtons_Config = { enabled = true };

--------------------------------------------------------------------------------------------------------
--                                                Main                                                --
--------------------------------------------------------------------------------------------------------

local f = CreateFrame("Frame","QButtons",UIParent);
f:SetBackdrop({ bgFile = "Interface\\ChatFrame\\ChatFrameBackground", edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", tile = 1, tileSize = 16, edgeSize = 16, insets = { left = 4, right = 4, top = 4, bottom = 4 } });
f:SetBackdropColor(0.1,0.22,0.35,1);
f:SetBackdropBorderColor(0.1,0.1,0.1,1);
f:EnableMouse(1);
f:SetMovable(1);
f:SetToplevel(1);
f:SetClampedToScreen(1);
f:SetFrameStrata("DIALOG");

local function OnMouseUp(self,button)
	f:StopMovingOrSizing();
	QButtons_Config.left, QButtons_Config.bottom = f:GetLeft(), f:GetBottom();
end

f:SetScript("OnMouseDown",function(self,button) f:StartMoving(); end);
f:SetScript("OnMouseUp",OnMouseUp);

f.tip = CreateFrame("GameTooltip","QButtonsTip",nil,"GameTooltipTemplate");
f.tip:SetOwner(UIParent,"ANCHOR_NONE");

local function SetEnabledStatus()
	if (QButtons_Config.enabled) then
		f:RegisterEvent("BAG_UPDATE");
		f:RegisterEvent("UNIT_INVENTORY_CHANGED");
		f:RegisterEvent("PLAYER_REGEN_ENABLE");
		if (not InCombatLockdown()) then
			f.UpdateButtons();
		end
	else
		f:UnregisterAllEvents();
		f:Hide();
	end
end

--------------------------------------------------------------------------------------------------------
--                                               Events                                               --
--------------------------------------------------------------------------------------------------------
local function OnEvent(self,event,...)
	-- Variables Loaded
	if (event == "VARIABLES_LOADED") then
		if (QButtons_Config.left and QButtons_Config.bottom) then
			f:SetPoint("BOTTOMLEFT",QButtons_Config.left,QButtons_Config.bottom);
		else
			f:SetPoint("CENTER");
		end
		if (QButtons_Config.enabled == nil) then
			QButtons_Config.enabled = true;
		end
		SetEnabledStatus()
	-- Other Events
	elseif (not InCombatLockdown()) then
		f.UpdateButtons();
	end
end

f:SetScript("OnEvent",OnEvent);
f:RegisterEvent("VARIABLES_LOADED");

--------------------------------------------------------------------------------------------------------
--                                            Command Line                                            --
--------------------------------------------------------------------------------------------------------
_G["SLASH_"..modName.."1"] = "/qb";
SlashCmdList[modName] = function(cmd)
	-- Extract Parameters
	local param1, param2 = cmd:match("^([^%s]+)%s*(.*)$");
	param1 = (param1 and param1:lower() or cmd:lower());
	-- Options
	if (param1 == "toggle") then
		QButtons_Config.enabled = not QButtons_Config.enabled;
		AzMsg("|2"..modName.."|r has now been |1"..(QButtons_Config.enabled and "enabled" or "disabled").."|r.");
		SetEnabledStatus();
	-- Invalid or No Command
	else
		UpdateAddOnMemoryUsage();
		AzMsg(format("----- |2%s|r |1%s|r ----- |1%.2f |2kb|r -----",modName,GetAddOnMetadata(modName,"Version"),GetAddOnMemoryUsage(modName)));
		AzMsg("The following |2parameters|r are valid for this addon:");
		AzMsg(" |2toggle|r = Toggles the mod being on or off");
	end
end
--------------------------------------------------------------------------------------------------------
--                                                Items                                               --
--------------------------------------------------------------------------------------------------------

f.items = {};

local function Button_OnEnter(self)
	GameTooltip:SetOwner(self,"ANCHOR_RIGHT");
	local bag, slot = self:GetAttribute("bag"), self:GetAttribute("slot");
	if (bag) then
		GameTooltip:SetBagItem(bag,slot);
	else
		GameTooltip:SetInventoryItem("player",slot);
	end
end

local function HideGTT()
	GameTooltip:Hide();
end

-- Make Loot Button
local function MakeItemButton()
	local b = CreateFrame("Button",nil,f,"SecureActionButtonTemplate");
	b:SetWidth(iconSize);
	b:SetHeight(iconSize);
	b:SetHighlightTexture("Interface\\Buttons\\ButtonHilight-Square");
	b:RegisterForClicks("LeftButtonUp","RightButtonUp");

	b:SetScript("OnEnter",Button_OnEnter);
	b:SetScript("OnLeave",HideGTT);

	b.icon = b:CreateTexture(nil,"ARTWORK");
	b.icon:SetAllPoints();

	b.count = b:CreateFontString(nil,"ARTWORK","NumberFontNormal");
	b.count:SetPoint("BOTTOMRIGHT",b.icon,-3,3);

	if (#f.items == 0) then
		b:SetPoint("TOPLEFT",f,8,-8);
	else
		b:SetPoint("LEFT",f.items[#f.items],"RIGHT",1,0);
	end

	tinsert(f.items,b);
	return b;
end

-- Add Button
local function AddButton(index,bag,slot,link,count)
	button = f.items[index] or MakeItemButton();

	button.icon:SetTexture(select(10,GetItemInfo(link)));
	button.count:SetText(count and count > 1 and count or "");

	button:SetAttribute("type*","item");
	button:SetAttribute("bag",bag);
	button:SetAttribute("slot",slot);

	button:Show();
end

-- Check Item
local function CheckItem(link)
	local itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, itemTexture = GetItemInfo(link);
	f.tip:ClearLines();
	f.tip:SetHyperlink(link);
	if (f.tip:NumLines() >= 3) and (_G["QButtonsTipTextLeft2"]:GetText() == ITEM_BIND_QUEST) then
		for i = 3, f.tip:NumLines() do
			if (_G["QButtonsTipTextLeft"..i]:GetText() or ""):find("^Use: .+") then
				return 1;
			end
		end
	end
	return;
end

-- Update Buttons
function f.UpdateButtons()
	local link, slotId;
	local index = 1;
	-- Inventory
	for bag = 0, NUM_BAG_FRAMES do
		for slot = 1, GetContainerNumSlots(bag) do
			link = GetContainerItemLink(bag,slot);
			if (link) and (CheckItem(link)) then
				AddButton(index,bag,slot,link,select(2,GetContainerItemInfo(bag,slot)));
				index = (index + 1);
			end
		end
	end
	-- Equipped Items
	for _, slotName in ipairs(slots) do
		slotId = GetInventorySlotInfo(slotName);
		link = GetInventoryItemLink("player",slotId);
		if (link) and (CheckItem(link)) then
			AddButton(index,nil,slotId,link);
			index = (index + 1);
		end
	end
	-- Hide/Show depending on we have items showing
	if (index == 1) then
		f:Hide();
	else
		f:Show();
		-- Hide the Rest
		for i = index, #f.items do
			f.items[i]:Hide();
		end
		-- Set Width
		f:SetWidth((iconSize + 1) * (index - 1) + 15);
		f:SetHeight(iconSize + 16);
	end
end

--------------------------------------------------------------------------------------------------------
--                                Global Chat Message Function (Rev 3)                                --
--------------------------------------------------------------------------------------------------------
if (not AZMSG_REV or AZMSG_REV < 3) then
	AZMSG_REV = 3;
	function AzMsg(text)
		DEFAULT_CHAT_FRAME:AddMessage(tostring(text):gsub("|1","|cffffff80"):gsub("|2","|cffffffff"),128/255,192/255,255/255);
	end
end